{% extends 'admin/model/list.html' %}
{% block body %}
  {{ super() }}
{% endblock %}
{% block list_row_actions %}
  {{ super() }}
  {% if row.orcid %}
    <a class="icon" href="{{ url_for('section', user_id=row.id, section_type='EMP', url=request.url) }}" title="Employment">
      <spam class="fa fa-briefcase glyphicon glyphicon-briefcase icon"></spam>
    </a>
    <a class="icon" href="{{ url_for('section', user_id=row.id, section_type='EDU', url=request.url) }}" title="Education">
      <spam class="fa fa-graduation-cap glyphicon glyphicon-education icon"></spam>
    </a>
  {% endif %}
{% endblock %}
