{% extends "layout.html" %}
{% block content %}
  <div class="container">
    <h2 class="display-2">Frequently Asked Questions</h2>
    <p>If you have questions that are not covered in this FAQ please contact us at orcid@royalsociety.org.nz</p>
    <div class="row">
      <div class="col-md-12">
        <a id="qlist"></a>
        <div>
          <ul>
            <li><a href="/faq#qwhat">What is the NZ ORCID Hub?</a></li>
            <li><a href="/faq#qhow">How do I get into the Hub?</a></li>
            <li><a href="/faq#qperm">How do I give my organisation permission to connect to my ORCID record?</a></li>
            <li><a href="/faq#qwhy">Why should I give my organisation permission to connect to my ORCID record?</a></li>
            <li><a href="/faq#qeduG">Can I access my ORCID record with my institutional log-in details?</a></li>
            <li><a href="/faq#qdise1">What if I don’t want my organisation to connect to my ORCID record?</a></li>
            <li><a href="/faq#qpriv1">Public, trusted-party, private - what do these mean?</a></li>
            <li><a href="/faq#qpriv2">Create and update activities - what counts as an activity?</a></li>
            <li><a href="/faq#qpriv3">Can I uncheck the box meaning permission is "this time only"?</a></li>
            <li><a href="/faq#qdise2">Can I give a lower level of permission than read trusted-party and write?</a></li>
            <li><a href="/faq#qdise3">How do I revoke my organisation's access to my ORCID record?</a></li>
            <li><a href="/faq#qdupe">Will the Hub write a duplicate if I already have an affiliation entry into my ORCID record?</a></li>
            <li><a href="/faq#qvani">My self-written affilation has more information and is much nicer - what can I do?</a></li>
            <li><a href="/faq#qnew">I don’t yet have an ORCID iD – can I create one from the Hub?</a></li>
            <li><a href="/faq#qfix">The information the Hub's writing about me is incorrect – how do I get this changed?</a></li>
            <li><a href="/faq#qntk">My organisation is not a member of Tuakiri – can I use the Hub?</a></li>
            <li><a href="/faq#qlea">Will I still be able to log into the Hub if I leave my organisation?</a></li>
            <li><a href="/faq#qdev">The Hub's changing a lot - how can I find out what's happening?</a></li>
          </ul>
        </div>
      </div>
    </div>
    <div class="row">
      <div class="col-md-12">
        <h3><a id="qwhat"></a>What is the NZ ORCID Hub?</h3>
        <p>The NZ ORCID Hub is a web service that assists the research organisations you're associated with to authoritatively assert
        these relationships in your ORCID record.  Whether you are a student, a member of staff, or (coming soon) have been a recipient
        of funding, these relationships can be written to your ORCID record, with the source of the information displayed as coming
        from the organisation involved. This benefits you as it proves that you have a bona fide affiliation with that institution.</p>
        <a href="/faq#qlist">Back to top</a>

        <h3><a id="qhow"></a>How do I get into the Hub?</h3>
        <p>Firstly, make sure the organisation you’re associated with is shown as using the Hub.  For a researcher belonging to a
        Tuakiri-member organisation, simply log in and use the Tuakiri Directory Service to select your home organisation where you'll
        be able to use your institutional credentials to log in and be returned, authenticated to the Hub.  For non-Tuakiri members,
        you'll first need to receive an email invitation from your organisation.  Once the Hub has been involved in writting to your
        ORCID record, your ORCID iD and organisational relationship is known and you're able to use the login via ORCID route.</p>
        <a href="/faq#qlist">Back to top</a>

        <h3><a id="qperm"></a>How do I give my organisation permission to connect to my ORCID record?</h3>
        <p>If you have logged into the Hub using your institutional credentials simply click the button to "Give Your Organisation
        Permission To Access Your ORCID Record" and follow the on-screen prompts.  The Hub uses the information reported about you
        by your organisation: if you're reported as being "staff" and/or "faculty" an employment affilation is written; if you're reported
        as being a "student", an education affilation is written; and if neither employment nor education can be detected the Hub
        will show you a warning.
        <br/>If you've come to the Hub through an email invitation, you've already given your organisation
        the necessary permissions.  Once you have done this, on signing into your ORCID record, you will see that your employment
        (or education if you are a student) now has an entry with the source of that information being your institution.</p>
        <a href="/faq#qlist">Back to top</a>

        <h3><a id="qwhy"></a>Why should I give my organisation permission to connect to my ORCID record?</h3>
        <p>Anyone can sign up for an ORCID iD and say that they are a student or staff at a respected organisation. By allowing your
        organisation to connect to your ORCID record and verify that you are, indeed, affiliated to that place, the source of this
        information in your record is more credible than if it is entered by you, manually.  By providing an authenticated ORCID iD
        to your organisation, you're making their work easier and can expect them to be putting processes in place to draw administrative
        content from your ORCID record to make your life easier as well.</p>
        <a href="/faq#qlist">Back to top</a>

        <h3><a id="qeduG"></a>Can I access my ORCID record with my institutional log-in details?</h3>
        <p>Not yet.<br/>The use of your home organisation’s login credentials to access services provided by another organisation
        is enabled by Federated Identity Management.  ORCID's ‘Institutional Account’  sign-in supports Federated Identity Management
        for members through the eduGAIN interfederation service.  Tuakiri, the New Zealand Access Federation, is in the process of
        becoming a full participating member of eduGAIN.  Once that happens, individuals at Tuakiri member organisations will be able
        to use their home credentials to access their ORCID records.</p>
        <a href="/faq#qlist">Back to top</a>

        <h3><a id="qdise1"></a>What if I don’t want my organisation to connect to my ORCID record, which I keep private?</h3>
        <p>Your ORCID record belongs to you!  If you don't want your organisation to connect to it, you are free to say "No" or to
        revoke permission if you've already given it. If, at any point, you wish to stop an organisation from accessing your ORCID
        record, simply log in to your ORCID record and, from the ‘Account Settings’ page, delete the organisation’s entry from your
        ‘Trusted Organisations’ list. Once removed, the organisation will no longer be able to read your limited-access content or
        write to your ORCID record until they gain your permission again.  If you want to remove yourself, and the record of your
        ORCID iD from the Hub, simply contact us and we'll delete your Hub record; please note however that information that you've
        made public in ORCID should be regarded as discoverable even if deleted or subsequently marked as private.</p>
        <a href="/faq#qlist">Back to top</a>

        <h3><a id="qpriv1"></a>Public, trusted-party, private - what do these mean?</h3>
        <p>ORCID allows three setting of privacy, with each item in your record able to have its privacy setting specified.  The three
        options are: public, i.e., anyone can see it; private, i.e., only you and the source can see it; and trusted-party, where the
        item is only visible to you, the source, and anyone you've given read-limited permission to.</p>
        <a href="/faq#qlist">Back to top</a>

        <h3><a id="qpriv2"></a>Create and update activities - what counts as an activity?</h3>
        <p>The Hub requests permission to create and update activities.  In the current ORCID schema (v2.0), activities encompass
        employment, education, funding and research works.  By granting this permission, you're allowing the organisation to write
        to all four sections of your ORCID record.</p>
        <a href="/faq#qlist">Back to top</a>

        <h3><a id="qpriv3"></a>Can I uncheck the box meaning permission is "this time only"?</h3>
        <p>The Hub requests long-term permission, as shown by the ticked box in the ORCID permission form stating "Unchecking this box
        will grant permission this time only".  Long-term permissions last for 20 years, and will allow your organsiation to update
        the items it writes without requiring your intervention.  Unchecking this box produces a permission that expires after an hour;
        i.e., to make changes after this, the organisation will need to get your permission again.</p>
        <a href="/faq#qlist">Back to top</a>

        <h3><a id="qdise2"></a>I'm happy for my organisation to know my ORCID iD, but don’t want it to write to my record –
        can I give a lower level of permission?</h3>
        <p>Your ORCID record really belongs to you!  The default level of permission requested through the Hub is a combined ability
        to both "read your trusted-party information" and "create or update activities", where activities covers your affiliations,
        funding and research works.  If, when asked by ORCID to "Authorize" or "Deny" the request, you select "Deny" you'll be returned
        to the Hub and given the option of granting lower level permissions, i.e., just "read your trusted-party information", and
        "know your ORCID iD".</p>
        <a href="/faq#qlist">Back to top</a>

        <h3><a id="qdise3"></a>How do I revoke my organisation's access to my ORCID record?</h3>
        <p>In your ORCID record, go to 'account settings' and delete your organisation from the list of 'trusted organisations'.
        This will also remove it from your Hub connections.  When you next visit your Hub profile while logged in with that organisation,
        you'll be offered the chance to "Give Your Organisation Permission To Access Your ORCID Record" again.</p>
        <a href="/faq#qlist">Back to top</a>

        <h3><a id="qdupe"></a>I have already put an entry into my ORCID record stating where I am employed (or where I am a student).
        If I go through the Hub and allow my organisation to connect with my record, will there now be two employment (education) records –
        one by me and one by my institution?</h3>
        <p>Yes, but like everything in your ORCID record you can choose which entry you make public.</p>
        <a href="/faq#qlist">Back to top</a>

        <h3><a id="qvani"></a>My self-written affiliation has more information and is much nicer than the Hub-written one – what can I do?</h3>
        <p>The default affiliation written by a Tuakiri-mediated login is rather plain, being only: the organisation, city, country, and
        whether it's an employment or education affiliation.  If your organisation provides a record detailing your affiliation in greater detail,
        e.g. your study course or job title, start and stop dates, etc, then the Hub is able to write this in place of the Tuakiri-mediated
        affiliation.  Feel free to contact us or your Technical Contact to find out if your organisation has plans to do this.</p>
        <a href="/faq#qlist">Back to top</a>

        <h3><a id="qnew"></a>I don’t yet have an ORCID iD– can I create one from the Hub?</h3>
        <p>Yes, definitely. Simply follow the on-screen prompt in the Hub to create an ORCID record when it asks you to sign in or create one.</p>
        <a href="/faq#qlist">Back to top</a>

        <h3><a id="qfix"></a>The information the Hub's writing about me is incorrect – how do I get this changed?</h3>
        <p>Contact your institution’s ORCID administrator. This will be the Technical Contact listed for your organisation on
        <a href="https://royalsociety.org.nz/orcid-in-new-zealand/new-zealand-orcid-consortium/who-is-involved-with-the-new-zealand-orcid-consortium/nz-orcid-consortium-members/">this page</a>.
        If your Technical Contact is not listed, please contact <a href="mailto:orcid@royalsociety.org.nz">orcid@royalsociety.org.nz</a>
        and we'll try to assist you</p>
        <a href="/faq#qlist">Back to top</a>

        <h3><a id="qntk"></a>My organisation is not a member of Tuakiri – can I use the Hub?</h3>
        <p>Yes. The Hub is designed to be of use to all NZ ORCID Consortium members.  The only requirement is that your organisation
        is actually using the Hub, and for non-Tuakiri members, you need to recieve an email invitation triggered by your organisation administrator in the Hub.
        <a href="/faq#qlist">Back to top</a>

        <h3><a id="qlea"></a>Will I still be able to log into the Hub if I leave my organisation, which uses Tuakiri?</h3>
        <p>This depends on how your organisation manages staff when they leave. If your organisation disables institution credentials, you will
        no longer be able to log into the Hub using your Tuakiri-mediated log in; however, if you've ever linked your ORCID iD to an organisation
        with the Hub you'll be able to log in with your ORCID record.</p>
        <a href="/faq#qlist">Back to top</a>

        <h3><a id="qdev"></a>The Hub's changing a lot - how can I find out what's happening?</h3>
        <p>If you’re interested in following the Hub’s progress, it’s an open development. Keep an eye on the resources available on the
        <a href="https://royalsociety.org.nz/orcid-in-new-zealand/new-zealand-orcid-consortium/who-is-involved-with-the-new-zealand-orcid-consortium/new-zealand-orcid-hub/">Hub page</a>
        at the Royal Society Te Apārangi’s website.  If you're interested in what's coming up, the Hub's <a href="https://test.orcidhub.org.nz">test environment</a>
        is where features we think are ready for exposure are shown.</p>
        <a href="/faq#qlist">Back to top</a>
      </div>
    </div>
  </div>
{% endblock %}
